<?php

// General helpers

/**
 * Contact whichever configured backend and get a new pay-to address
 *
 * @param string $label
 * @return bool|mixed
 */
function uc_bitcoin_get_new_pay_to_address($label = "") {

  $address = FALSE;

  switch (variable_get('uc_bitcoin_conn_type', UC_BITCOIN_DEFAULT_SERVER_TYPE)) {
    case 'Bitcoin daemon':
      // see if the connection works
      module_load_include('inc', 'uc_bitcoin', 'includes/bitcoin-php/bitcoin');
      $bitcoin_client = new BitcoinClient(variable_get("uc_bitcoin_server_scheme", ""),
        variable_get("uc_bitcoin_server_username", ""),
        variable_get("uc_bitcoin_server_password", ""),
        variable_get("uc_bitcoin_server_address", ""),
        variable_get("uc_bitcoin_server_port", ""),
        variable_get("uc_bitcoin_server_certificate_path", ""));

      if (TRUE !== ($fault = $bitcoin_client->can_connect())) {
        watchdog('uc_bitcoin_order', 'Could not connect to backend');
        break;
      }

      try {
        $address = $bitcoin_client->query("getnewaddress", $label);
      } catch (BitcoinClientException $e) {
        watchdog('uc_bitcoin_order', 'getnewaddress failed. Fault: !fault', array('!fault' => $e->getMessage()));
      }

      break;

    case 'Blockchain.info':
      // Do a balance check
      $guid = urlencode(variable_get('uc_bitcoin_blockchain_guid',''));
      $main_password = urlencode(variable_get('uc_bitcoin_blockchain_main_password',''));
      $second_password = urlencode(variable_get('uc_bitcoin_blockchain_second_password',''));
      $label = urlencode(trim($label));
      $url = "/merchant/{$guid}/new_address?password={$main_password}&second_password={$second_password}&label={$label}";

      // Autoload on demand! Your work: zero.
      $client = new Guzzle\Http\Client('https://blockchain.info');
      $request = $client->get($url);
      $response = $request->send();

      $status = $response->getStatusCode();
      if ($status == 200) {
        $result = json_decode($response->getBody());
        if(isset($result->error)) {
          watchdog('uc_bitcoin_order', 'Could not get new_address, Password or something else was wrong? (!error)', array('!error'=>$result->error));
        } else if(isset($result->address)) {
          $address = $result->address;
        }
      }
      else {
        watchdog('uc_bitcoin_order', 'Could not get new_address, some kind of network issue?');
      }
      break;
  }
  return $address;
}


/**
 * Convert an orders balance to BTC
 *
 * @param $order
 */
function uc_bitcoin_payment_amount_to_btc($order) {

  $rate = FALSE;

  // Should be 8 decimal places https://en.bitcoin.it/wiki/FAQ
  $precision = variable_get('uc_bitcoin_btc_precision', 8);
  $round_type = variable_get('uc_bitcoin_btc_rounding', PHP_ROUND_HALF_UP);

  if (!empty($order->bitcoin_payment)) {
    $rate = $order->bitcoin_payment['exchange_rate'];
  }
  else {
    $rate = uc_bitcoin_exchange_rate($order->currency);
  }

  if ($rate) {
    return round($order->order_total / $rate, $precision, $round_type);
  }

  return FALSE;
}


/**
 * UberCart currency convert, do a lookup to MtGox and see what the rate is
 *
 * @param $order ubercart order
 * @return float amount
 */

/**
 * UberCart currency convert, do a lookup to MtGox and see what the rate is
 *
 * @param $order ubercart order
 * @return float amount
 */
function uc_bitcoin_exchange_rate($currency = 'USD') {
  static $data;
  $rate = FALSE;

  if ($currency == 'BTC') {
    // BTC 1:1
    return 1;
  }

  if (isset($data[$currency])) {
    return $data[$currency];
  }

  $preferred = variable_get('uc_bitcoin_rate_lookup', UC_BITCOIN_DEFAULT_RATE_LOOKUP);
  $cid = 'uc_bitcoin_'.$preferred;
  switch ($preferred) {
    case 'coinbase':
      // caching
      $cache = cache_get($cid);
      if (!$cache || (isset($cache) && time() > $cache->expire)) {
        if ($response = drupal_http_request('https://api.coinbase.com/v1/currencies/exchange_rates')) {
          if ($response->code == 200) {
            cache_set($cid, $response, 'cache', time() + variable_get('uc_bitcoin_x_seconds', 120));
          }
        }
      }
      else {
        $response = $cache->data;
      }
      // lookup
      $currency_code = 'btc_to_' . strtolower($currency);
      if ($response && $response->code == 200) {
        $result = json_decode($response->data, TRUE);
        if (isset($result[$currency_code]) && $result[$currency_code] > 0) {
          $rate = $result[$currency_code];
        }
      }

      break;
    case 'blockchain.info':
      // caching
      $cache = cache_get($cid);
      if (!$cache || (isset($cache) && time() > $cache->expire)) {
        if ($response = drupal_http_request('https://blockchain.info/ticker')) {
          if ($response->code == 200) {
            cache_set($cid, $response, 'cache', time() + variable_get('uc_bitcoin_x_seconds', 120));
          }
        }
      }
      else {
        $response = $cache->data;
      }

      // lookup
      if ($response && $response->code == 200) {
        $result = json_decode($response->data, TRUE);
        if (isset($result[$currency]) && is_array($result[$currency]) && $result[$currency]['last'] > 0) {
          $rate = $result[$currency]['last'];
        }
      }
      break;
  }

  if($rate) {
    $data[$currency] = $rate;
  } else {
    watchdog('uc_bitcoin', 'CoinBase lookup - Unable to get current exchange rate, try refresh', array(), WATCHDOG_ERROR);
  }


  return $rate;
}
